@@ -16,9 +16,9 @@
 #' @param url the character host or dispatcher URL to dial into, including the
 #'   port to connect to, e.g. 'tcp://hostname:5555' or
 #'   'tls+tcp://10.75.32.70:5555'.
-#' @param ... reserved but not currently used.
-#' @param dispatcher \[default FALSE\] logical value, which should be set to
+#' @param dispatcher \[default TRUE\] logical value, which should be set to
 #'   TRUE if using dispatcher and FALSE otherwise.
+#' @param ... reserved but not currently used.
 #' @param asyncdial \[default FALSE\] whether to perform dials asynchronously.
 #'   The default FALSE will error if a connection is not immediately possible
 #'   (for instance if [daemons()] has yet to be called on the host, or the
@@ -26,10 +26,10 @@
 #'   (indefinitely) if not immediately successful, which is more resilient but
 #'   can mask potential connection issues.
 #' @param autoexit \[default TRUE\] logical value, whether the daemon should
-#'   exit automatically when its socket connection ends. If a signal from the
-#'   \pkg{tools} package, such as `tools::SIGINT`, or an equivalent integer
-#'   value is supplied, this signal is additionally raised on exit (see
-#'   'Persistence' section below).
+#'   exit automatically when its socket connection ends. By default, the process
+#'   ends immediately when the host process ends. Supply `NA` to have a daemon
+#'   complete any tasks in progress before exiting (see 'Persistence' section
+#'   below).
 #' @param cleanup \[default TRUE\] logical value, whether to perform cleanup of
 #'   the global environment and restore attached packages and options to an
 #'   initial state after each evaluation.
@@ -54,7 +54,7 @@
 #'   certificate authority certificate chain and \[ii\] the empty string `""`.
 #' @param rs \[default NULL\] the initial value of .Random.seed. This is set
 #'   automatically using L'Ecuyer-CMRG RNG streams generated by the host process
-#'   and should not be independently supplied.
+#'   if applicable, and should not be independently supplied.
 #'
 #' @return Invisibly, an integer exit code: 0L for normal termination, and a
 #'   positive value if a self-imposed limit was reached: 1L (idletime), 2L
@@ -63,14 +63,14 @@
 #' @section Persistence:
 #'
 #' The `autoexit` argument governs persistence settings for the daemon. The
-#' default TRUE ensures that it will exit cleanly once its socket connection
-#' has ended.
+#' default TRUE ensures that it will exit as soon as its socket connection
+#' with the host process drops.
 #'
-#' Instead of TRUE, supplying a signal from the \pkg{tools} package, such as
-#' `tools::SIGINT`, or an equivalent integer value, sets this signal to be
-#' raised when the socket connection ends. For instance, supplying SIGINT allows
-#' a potentially more immediate exit by interrupting any ongoing evaluation
-#' rather than letting it complete.
+#' Supplying `NA` will allow a daemon to exit cleanly once its socket connection
+#' with the host process drops, as soon as it has finished any task that is
+#' currently in progress. This may be useful if the daemon is performing some
+#' side effect such as writing files to disk, and the result is not required in
+#' the host process.
 #'
 #' Setting to FALSE allows the daemon to persist indefinitely even when there is
 #' no longer a socket connection. This allows a host session to end and a new
@@ -82,7 +82,7 @@
 #'
 daemon <- function(
   url,
-  dispatcher = FALSE,
+  dispatcher = TRUE,
   ...,
   asyncdial = FALSE,
   autoexit = TRUE,
@@ -98,13 +98,12 @@ daemon <- function(
   cv <- cv()
   sock <- socket(if (dispatcher) "poly" else "rep")
   on.exit(reap(sock))
-  autoexit && pipe_notify(sock, cv, remove = TRUE, flag = autoexit)
+  pipe_notify(sock, cv, remove = TRUE, flag = flag_value_auto(autoexit))
   if (length(tls)) tls <- tls_config(client = tls)
-  dial_and_sync_socket(sock, url, autostart = asyncdial || NA, tls = tls)
+  dial_sync_socket(sock, url, autostart = asyncdial || NA, tls = tls)
 
   `[[<-`(., "sock", sock)
   on.exit(`[[<-`(., "sock", NULL), add = TRUE)
-  if (is.numeric(rs)) `[[<-`(.GlobalEnv, ".Random.seed", as.integer(rs))
   if (!output) {
     devnull <- file(nullfile(), open = "w", blocking = FALSE)
     sink(file = devnull)
@@ -118,7 +117,6 @@ daemon <- function(
       add = TRUE
     )
   }
-  snapshot()
   xc <- 0L
   task <- 1L
   timeout <- if (idletime > walltime) walltime else if (is.finite(idletime)) idletime
@@ -126,11 +124,12 @@ daemon <- function(
 
   if (dispatcher) {
     aio <- recv_aio(sock, mode = 1L, cv = cv)
-    if (is.numeric(id))
-      send(sock, c(0L, as.integer(id)), mode = 2L, block = TRUE)
+    is.numeric(id) && send(sock, c(0L, as.integer(id)), mode = 2L, block = TRUE)
     wait(cv) || return(invisible(xc))
-    serial <- collect_aio(aio)
-    if (is.list(serial)) `opt<-`(sock, "serial", serial)
+    bundle <- collect_aio(aio)
+    `[[<-`(.GlobalEnv, ".Random.seed", if (is.numeric(rs)) as.integer(rs) else bundle[[1L]])
+    if (is.list(bundle[[2L]])) `opt<-`(sock, "serial", bundle[[2L]])
+    snapshot()
     repeat {
       aio <- recv_aio(sock, mode = 1L, timeout = timeout, cv = cv)
       wait(cv) || break
@@ -156,6 +155,8 @@ daemon <- function(
       task <- task + 1L
     }
   } else {
+    if (is.numeric(rs)) `[[<-`(.GlobalEnv, ".Random.seed", as.integer(rs))
+    snapshot()
     repeat {
       ctx <- .context(sock)
       aio <- recv_aio(ctx, mode = 1L, timeout = timeout, cv = cv)
@@ -194,7 +195,7 @@ daemon <- function(
   cv <- cv()
   sock <- socket("rep")
   on.exit(reap(sock))
-  pipe_notify(sock, cv, remove = TRUE)
+  pipe_notify(sock, cv, remove = TRUE, flag = flag_value())
   dial(sock, url = url, autostart = NA, fail = 2L)
   `[[<-`(., "sock", sock)
   data <- eval_mirai(recv(sock, mode = 1L, block = TRUE))
@@ -225,7 +226,7 @@ eval_mirai <- function(._mirai_.) {
   )
 }
 
-dial_and_sync_socket <- function(sock, url, autostart = NA, tls = NULL) {
+dial_sync_socket <- function(sock, url, autostart = NA, tls = NULL) {
   cv <- cv()
   pipe_notify(sock, cv, add = TRUE)
   dial(sock, url = url, autostart = autostart, tls = tls, fail = 2L)
@@ -243,3 +244,8 @@ do_cleanup <- function() {
 
 snapshot <- function()
   `[[<-`(`[[<-`(`[[<-`(., "op", .Options), "se", search()), "vars", names(.GlobalEnv))
+
+flag_value_auto <- function(autoexit)
+  { isFALSE(autoexit) || isNamespace(topenv(parent.frame(), NULL)) } && return(autoexit) || is.na(autoexit) || isNamespaceLoaded("covr") || return(tools::SIGTERM)
+
+flag_value <- function() isNamespaceLoaded("covr") || return(tools::SIGTERM)
